import re, logging

def process(doc,somethinglist:list,imgpath :str):
    """
    doc Document文件对象   
    something list

    >>>[{
        "key":"锚点标记",
        "type":"text/img/tab/br",
        "value":"文本"/[img_path,width,height]/[rows,cols]
        },
        ...]
    """
    # 遍历一遍
    for paragraph in doc.paragraphs:
        # 如果碰到锚点
        rer = re.findall(r"{{.*?}}",paragraph.text)
        if rer != None:
            # 根据锚点信息
            for i in rer:
                # 查找输入字典
                sth = list(filter(lambda x:x["key"] == i.strip(r"{{").strip(r"}}"),somethinglist))
                # 字典中没有这条锚点信息
                if sth == []:
                    logging.warning(f"缺失模板信息 {i} 的key，跳过")
                    continue
                
                sth = sth[0]
                # 映射出输入信息中的字段
                # ikey = sth["key"]
                itype = sth["type"]
                ivalue = sth["value"]

                # 判断输入是什么类型
                TEXT = (itype == "text")
                PICTURE = (itype == ("img" or "picture"))
                TAB = (itype == ("tab" or "table"))
                BREAK = (itype == ("br" or "break"))

                # 替换文本，对一段文字也有效
                if TEXT:
                    # paragraph.runs[-1].add_text(something["value"])
                    paragraph.text = re.sub(i,ivalue,paragraph.text)
                    continue
                
                # 删除锚点文本
                paragraph.text = re.sub(i,"",paragraph.text)

                # 在末尾添加图片
                # params:
                # [img_path,width,height]
                # todo 碰到http开头的自动下载并插入
                if PICTURE:
                    paragraph.add_run().add_picture(*ivalue)
                    # paragraph.runs[-1].add_picture(path)
                
                # 在末尾添加表格
                # params:
                # [rows,cols]
                if TAB:
                    paragraph.add_run().add_tab(*ivalue)

                # 添加换行
                if BREAK:
                    paragraph.add_run().add_break()
    return doc





from PyQt5.QtCore import QUrl
from PyQt5.QtGui import QDesktopServices, QIcon, QIconEngine
from PyQt5.QtWidgets import QMessageBox, QMainWindow, QApplication, QListWidgetItem, QInputDialog, QFileDialog
from .form import Ui_MainWindow
import httpx, json, os
from fastdocx import WordCore
from .style import stype
# class CommonHelper:
#   def __init__(self):
#     pass
 
#   @staticmethod
#   def readQss(style):
#     with open(style, 'r') as f:
#         return f.read()

# todo:获取输出、改造为迭代器提高效率
class item(QListWidgetItem):
  def __init__(self, name :str, icon: str, id: str, author: str, version: str, config: str, description: str,tmpdir: str, parent = None):
    super(item, self).__init__(parent)
    self.tmpdir = tmpdir
    self.setText(name)
    iconame = icon.split("/")[-1]
    with httpx.stream("GET", icon) as response:
      with open(self.tmpdir+iconame,"wb+") as f:
          for chunk in response.iter_bytes():
              f.write(chunk)
    self.setIcon(QIcon(self.tmpdir+iconame))
    self.setToolTip(f"ID:{id}\n作者:{author}\n版本:{version}")
    self.config = config
    self.description = description
    self.name = name
    self.version = version
    self.author = author
  
  def __next__():
    pass
  
  def __iter__():
    pass

  
class fastdocx(QMainWindow, Ui_MainWindow):
    def __init__(self,tmpdir:str = "./tmp/",source_url:str = "https://v.gonorth.top:444/file/index.json", parent = None):
        super(fastdocx, self).__init__(parent)
        # 重连10次
        self._time = 10
        self.source_url = source_url
        self.tmpdir = tmpdir
        self.setupUi(self)
        try:
          self.setWindowIcon(QIcon('icon.ico'))
        except:
          pass
        self.setWindowTitle('FastDocx')
        self.workdirButton.clicked.connect(self.workdirButtonClicked)
        self.process.clicked.connect(self.startProcess)
        self.listWidget.itemClicked.connect(self.setDetails)
        self.source.triggered.connect(self.setSourse)
        self.about.triggered.connect(self.aboutOpenWeb)
        # self.myinit()

    # todo:修复打开bug
    def aboutOpenWeb(self):
      QDesktopServices.openUrl(QUrl("https://github.com/sovlookup"))

    def setDetails(self, item):
      self.name.setText(item.name)
      self.author.setText(item.author)
      self.version.setText(item.version)
      self.description.setText(item.description)
      self.config = item.config
    
    def setSourse(self):
      text, ok = QInputDialog.getText(self,"自定义源地址","设置源地址:")
      if ok and str(text).startswith("http"):
        self.source_url = str(text)
        
    def workdirButtonClicked(self):
      dir = QFileDialog.getExistingDirectory(self, "输出文件夹", "./") 
      self.workdir.setText(dir)
      self.word = WordCore(dir)

    def startProcess(self):
      self.process.setText("处理中...")
      try:
        status = self.word.load(self.config).process()
        if status:
          QMessageBox.information(self,"成功","运行成功请查看输出目录!")
      except AttributeError:
        QMessageBox.warning(self,"检查","请选择任务和输出文件夹！")
      except httpx.ConnectTimeout:
        try:
          self.word.load(self.config).process()
        except httpx.ConnectTimeout:
          QMessageBox.warning(self,"超时","请检查网络连接")
      finally:
        self.process.setText("开始任务")


    def myinit(self):
      if os.path.exists(self.tmpdir) == False:
        os.makedirs(self.tmpdir)
      for item in self.download():
        self.listWidget.addItem(item)

    def download(self):
      try:
        source = json.loads(httpx.get(self.source_url).content)

        for k,v in source.items():
          yield item("\n"+v.get("taskname")+"\n",v.get("icon"),k,v.get("author"),v.get("version"),v.get("config"),v.get("description"),self.tmpdir)
      except httpx.ConnectTimeout:
        if self._time == 0:
          pass
        self.download()
        self._time -= 1


def ui():
  """可视化界面
  """
  app = QApplication([])
  widget = fastdocx()
  # qssStyle = CommonHelper.readQss('./qss/black.qss')
  widget.setStyleSheet(stype)
  widget.show()
  widget.myinit()
  app.exec_()





stype = """
/* === Shared === */
QStackedWidget, QLabel, QPushButton, QRadioButton, QCheckBox, 
QGroupBox, QStatusBar, QToolButton, QComboBox, QDialog {
    background-color: #222222;
    color: #BBBBBB;
    font-family: "Segoe UI";
}

/* === QWidget === */
QWidget:window {
    background: #222222;
    color: #BBBBBB;
    font-family: "Segoe UI";
}

/* === QToolTip === */
QToolTip {
    background-color: #000000;
    border: 2px solid #333333;
    color: antiquewhite;
}

/* === QPushButton === */
QPushButton {
    border: 1px solid #333333;
    padding: 4px;
    min-width: 65px;
    min-height: 12px;
}

QPushButton:hover {
    background-color: #333333;
    border-color: #444444;
}

QPushButton:pressed {
    background-color: #111111;
    border-color: #333333;
    color: antiquewhite;
}

QPushButton:disabled {
    color: #333333;
}

/* === Checkable items === */
QCheckBox::indicator, QRadioButton::indicator, QTreeView::indicator {
    width: 16px;
    height: 16px;
    background-color: #111111;
    border: 1px solid #333333;
}

QRadioButton::indicator {
    border-radius: 8px;
}

QCheckBox::indicator::checked, QRadioButton::indicator::checked, QTreeView::indicator::checked {
    background-color: qradialgradient(cx:0.5, cy:0.5, fx:0.25, fy:0.15, radius:0.3, stop:0 #BBBBBB, stop:1 #111111);
}

QCheckBox::indicator:disabled, QRadioButton::indicator:disabled, QTreeView::indicator:disabled {
    background-color: #444444;
}

QCheckBox::indicator::checked:disabled, QRadioButton::indicator::checked:disabled, QTreeView::indicator::checked:disabled {
    background-color: qradialgradient(cx:0.5, cy:0.5, fx:0.25, fy:0.15, radius:0.3, stop:0 #BBBBBB, stop:1 #444444);
}

/* === QComboBox === */
QComboBox {
    background-color: black;
    border: 1px solid #333333;
    color: white;
    padding:1px 2em 1px 3px;
}

QComboBox::drop-down {
    subcontrol-origin: padding;
    subcontrol-position: top right;
    border-left: 1px solid #333333;
}

QComboBox::down-arrow {
    border: 2px solid #333333;
    width: 6px;
    height: 6px;
    background: #5f5f5f;
}

/* === QGroupBox === */
QGroupBox {
    border: 2px solid #333333;
    margin-top: 2ex;
}

QGroupBox::title {
    color: antiquewhite;
    subcontrol-origin: margin;
    subcontrol-position: top left;
    margin-left: 5px;
}

/* === QTabWidget === */
QTabWidget::pane {
    background: #222222;
    border: 2px solid #333333;
}

/* === QTabBar === */
QTabBar::tab {
    background: transparent;
    border: 1px solid #333333;
    border-bottom: none;
    color: #BBBBBB;
    padding-left: 5px;
    padding-right: 10px;
    padding-top: 3px;
    padding-bottom: 3px;
}

QTabBar::tab:hover {
    background-color: #333333;
    border: 1px solid #444444;
    border-bottom: none;
}

QTabBar::tab:selected {
    background-color: #111111;
    border: 1px solid #333333;
    border-top: 1px solid antiquewhite;
    border-bottom: none;
    color: antiquewhite
}

/* === QToolBar === */
QToolBar {
    background-color: #222222;
    border: none;
    padding: 1px;
}

QToolBar:handle {
    background: #222222;
    border-left: 1px dotted antiquewhite;
    color: #BBBBBB;
}

QToolBar::separator {
    width: 6px;
    background-color: #222222;
}

/* === QToolButton === */
QToolButton {
    border: 1px solid #333333;
    margin: 1px;
}

QToolButton:hover {
    background-color: #333333;
    border: 1px solid #444444;
}

QToolButton[popupMode="1"] { /* only for MenuButtonPopup */
    padding-right: 20px; /* make way for the popup button */
}

QToolButton::menu-button {
    border-left: 1px solid #333333;
    background: transparent;
    width: 16px;
}

QToolButton::menu-button:hover {
    border-left: 1px solid #444444;
    background: transparent;
    width: 16px;
}

QToolButton:checked, QToolButton:pressed {
    background-color: #111111;
    color: antiquewhite;
}

/* === QMenu === */
QMenu {
    background-color: black;
    border: 1px solid gray;
    color: white;
    padding: 1px;
}

QMenu::item {
    padding: 2px 25px 2px 20px;
    border: 1px solid transparent;
}

QMenu::item:disabled {
    color: #666666;
}

QMenu::item:selected {
    border-color: gray;
    background: #222222;
}

QMenu::icon:checked {

}

QMenu::separator {
    height: 1px;
    background: #222222;
    margin-left: 10px;
    margin-right: 10px;
    margin-top: 1px;
    margin-bottom: 1px;
}

QMenu::indicator {
    width: 13px;
    height: 13px;
}

/* === QMenuBar === */
QMenuBar {
    background-color: black;
    color: white;
}

QMenuBar::item {
    background: transparent;
}

QMenuBar::item:disabled {
    color: gray;
}

QMenuBar::item:selected {
    background: #222222;
}

QMenuBar::item:pressed {
    background: #444444;
}
 
/* === QScrollBar:vertical === */
QScrollBar:vertical {
    background: #111111;
    width: 16px;
    margin: 16px 0 16px 0;
}

QScrollBar::handle:vertical {
    background: #555555;
    min-height: 16px;
}

QScrollBar::add-line:vertical {
    background: #444444;
    height: 16px;
    subcontrol-position: bottom;
    subcontrol-origin: margin;
}

QScrollBar::sub-line:vertical {
    background: #444444;
    height: 16px;
    subcontrol-position: top;
    subcontrol-origin: margin;
}

QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {
    background: none;
}

QScrollBar:up-arrow:vertical, QScrollBar:down-arrow:vertical {
    border: 2px solid #333333;
    width: 6px;
    height: 6px;
    background: #5f5f5f;
}

/* === QScrollBar:horizontal === */
QScrollBar:horizontal {
    background: #111111;
    height: 16px;
    margin: 0 16px 0 16px;
}

QScrollBar::handle:horizontal {
    background: #555555;
    min-width: 16px;
}

QScrollBar::add-line:horizontal {
    background: #444444;
    width: 16px;
    subcontrol-position: right;
    subcontrol-origin: margin;
}

QScrollBar::sub-line:horizontal {
    background: #444444;
    width: 16px;
    subcontrol-position: left;
    subcontrol-origin: margin;
}

QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal {
    background: none;
}

QScrollBar:left-arrow:horizontal, QScrollBar:right-arrow:horizontal {
    border: 2px solid #333333;
    width: 6px;
    height: 6px;
    background: #5f5f5f;
}

/* =================== */
QLineEdit, QListView, QTreeView, QTableView, QAbstractSpinBox {
    background-color: black;
    color: #BBBBBB;
    border: 1px solid #333333;
}

QAbstractScrollArea, QLineEdit, QTextEdit, QAbstractSpinBox, QComboBox {
    border-color: #333333;
    border: 1px solid #333333;

}

/* === QHeaderView === */
QHeaderView::section {
    background: #222222;
    border: 0;
    color: #BBBBBB;
    padding: 3px 0 3px 4px;
}

/* === QListView === */
QListView::item:hover {
    background: #333333;
}

QListView::item:selected {
    background: #111111;
    color: antiquewhite;
}

/* === QTableView === */
QTableView::item:hover {
    background: #333333;
}

QTableView::item:hover {
    background: #111111;
    color: antiquewhite;
}

/* === QTreeView === */
QTreeView::item {
    background: black;
}

QTreeView::item:hover {
    background: #333333;
}

QTreeView::item:selected {
    background: #111111;
    color: antiquewhite;
}

QTreeView::branch {

}

QTreeView::branch:has-siblings:adjoins-item {

}

QTreeView::branch:has-siblings:!adjoins-item {

}

QTreeView::branch:closed:has-children:has-siblings {

}

QTreeView::branch:has-children:!has-siblings:closed {

}

QTreeView::branch:!has-children:!has-siblings:adjoins-item {

}

QTreeView::branch:open:has-children:has-siblings {

}

QTreeView::branch:open:has-children:!has-siblings {

}

/* === Customizations === */
QFrame#infoLabel {
    border: 1px inset #333333;
}
2.

.QWidget {
   background-color: beige;
}

QToolBar {
    background-color: beige;
}

QDialog, QFileDialog {
    background-color: beige;
}

QTabWidget::pane { /* The tab widget frame */
    border-top: 2px solid #C2C7CB;
}

QTabWidget::tab-bar {
    left: 5px; /* move to the right by 5px */
}

QTabBar, QTabWidget {
    background-color: beige;
}
QTabBar::tab {
     background: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
                                 stop: 0 #E1E1E1, stop: 0.4 #DDDDDD,
                                 stop: 0.5 #D8D8D8, stop: 1.0 #D3D3D3);
     border: 1px solid darkkhaki;
     border-bottom-color: #C2C7CB; /* same as the pane color */
     border-top-left-radius: 4px;
     border-top-right-radius: 4px;
     min-width: 8ex;
     padding: 2px;
 }
QTabBar::tab:selected, QTabBar::tab:hover {
    background: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
                                stop: 0 #fafafa, stop: 0.4 #f4f4f4,
                                stop: 0.5 #e7e7e7, stop: 1.0 #fafafa);
}

QTabBar::tab:selected {
    border-color: #9B9B9B;
    border-bottom-color: #C2C7CB; /* same as pane color */
}

QTabBar::tab:!selected {
    margin-top: 2px; /* make non-selected tabs look smaller */
}

/* Nice Windows-XP-style password character. */
QLineEdit[echoMode="2"] {
    lineedit-password-character: 9679;
}

QHeaderView::section {
     background-color: qlineargradient(x1:0, y1:0, x2:0, y2:1,
                                       stop:0 #616161, stop: 0.5 #505050,
                                       stop: 0.6 #434343, stop:1 #656565);
     color: white;
     padding-left: 4px;
     border: 1px solid #6c6c6c;
 }

 QHeaderView::section:checked
 {
     background-color: red;
 }


/* We provide a min-width and min-height for push buttons
   so that they look elegant regardless of the width of the text. */
QPushButton {
    background-color: palegoldenrod;
    border-width: 2px;
    border-color: darkkhaki;
    border-style: solid;
    border-radius: 5;
    padding: 3px;
    min-width: 9ex;
    min-height: 2.5ex;
}

QPushButton:hover {
   background-color: khaki;
}

/* Increase the padding, so the text is shifted when the button is
   pressed. */
QPushButton:pressed {
    padding-left: 5px;
    padding-top: 5px;
    background-color: #d0d67c;
}

QLabel, QAbstractButton {
    font: bold;
}

/* Mark mandatory fields with a brownish color. */
.mandatory {
    color: brown;
}

/* Bold text on status bar looks awful. */
QStatusBar QLabel {
   font: normal;
}

QStatusBar::item {
    border-width: 1;
    border-color: darkkhaki;
    border-style: solid;
    border-radius: 2;
}

QStackedWidget, QComboBox, QLineEdit, QSpinBox, QTextEdit, QListView, QWebView, QTreeView, QHeaderView {
    background-color: cornsilk;
    selection-color: #0a214c; 
    selection-background-color: #C19A6B;
}

QListView {
    show-decoration-selected: 1;
}

QListView::item:hover {
    background-color: wheat;
}

/* We reserve 1 pixel space in padding. When we get the focus,
   we kill the padding and enlarge the border. This makes the items
   glow. */
QLineEdit, QFrame {
    border-width: 1px;
    padding: 1px;
    border-style: solid;
    border-color: darkkhaki;
    border-radius: 5px;
}

/* As mentioned above, eliminate the padding and increase the border. */
QLineEdit:focus, QFrame:focus {
    border-width: 3px;
    padding: 0px;
}

/* A QLabel is a QFrame  */
QLabel {
    border: none;
    padding: 0;
    background: none;
}

/* A QToolTip is a QLabel  */
QToolTip {
    border: 2px solid darkkhaki;
    padding: 5px;
    border-radius: 3px;
    opacity: 200;
}

/* Nice to have the background color change when hovered. */
QRadioButton:hover, QCheckBox:hover {
    background-color: wheat;
}

/* Force the dialog's buttons to follow the Windows guidelines. */
QDialogButtonBox {
    button-layout: 0;
}


3.
/*
    Style by evilworks, 2012-2013. pollux@lavabit.com
    This file is Public Domain.
*/

/* === Shared === */
QStackedWidget, QLabel, QPushButton, QRadioButton, QCheckBox, 
QGroupBox, QStatusBar, QToolButton, QComboBox, QDialog, QTabBar {
    font-family: "Segoe UI";
    background-color: #888;
    color: #000;
}

/* === QWidget === */
QWidget:window {
    font-family: 'Segoe UI';
    background-color: #888;
}

/* === QPushButton === */
QPushButton {
    border: 1px solid #555;
    padding: 4px;
    min-width: 65px;
    min-height: 12px;
}

QPushButton:hover {
    background-color: #999;
}

QPushButton:pressed {
    background-color: #333;
    border-color: #555;
    color: #AAA;
}

QPushButton:disabled {
    color: #333333;
}

/* === QComboBox === */
QComboBox {
    background-color: #AAA;
    border: 1px solid #555;
    color: black;
}

QComboBox::drop-down {
    subcontrol-origin: padding;
    subcontrol-position: top right;
    border-left: 1px solid #333333;
}

/* === QGroupBox === */
QGroupBox {
    border: 1px solid #555;
    margin-top: 2ex;
}

QGroupBox::title {
    color: black;
    subcontrol-origin: margin;
    subcontrol-position: top left;    
    border: 1px solid #555;
}

/* === QTabBar === */
QTabBar::tab {
    border-bottom: none;
    color: #000;
    padding: 4px;
    background-color: #888;
    border: 1px solid #555;
}

QTabBar::tab:hover {
    background-color: #AAA;
}

QTabBar::tab:selected {
    background-color: #000;
    color: white;
}

/* === QTabWidget === */
QTabWidget::pane {
    background: #888;
    border: 1px solid #555;
}


/* === QToolBar === */
QToolBar {
    background: #949494;
    border: none;
    padding-left: 0px;
    padding-right: 0px;
    margin: 2px;
}

QToolBar::separator {
    width: 1px;
    margin-left: 3px;
    margin-right: 3px;
    background-color: #555;
}

/* === QToolButton === */
QToolButton {
    border: 1px solid #666;
    margin: 1px;
}

QToolButton:hover {
    background-color: #AAA;
}

QToolButton[popupMode="1"] { /* only for MenuButtonPopup */
    padding-right: 20px; /* make way for the popup button */
}

QToolButton::menu-button {
    border-left: 1px solid #666;
    background: transparent;
    width: 16px;
}

QToolButton::menu-button:hover {
    border-left: 1px solid #666;
    background: transparent;
    width: 16px;
}

QToolButton:checked, QToolButton:pressed {
    background-color: #000;
    border: 1px solid #555;
    color: white;
}

/* === QScrollBar:vertical === */
QScrollBar:vertical {
    width: 16px;
    margin: 16px 0 16px 0;
    background: #333;
}

QScrollBar::handle:vertical {
    background: #888;
    min-height: 16px;
    border-top: 1px solid #666;
    border-bottom: 1px solid #666;
}

QScrollBar::add-line:vertical {
    background: #888;
    height: 16px;
    subcontrol-position: bottom;
    subcontrol-origin: margin;
}

QScrollBar::sub-line:vertical {
    background: #888;
    height: 16px;
    subcontrol-position: top;
    subcontrol-origin: margin;
}

QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {
    background: none;
}

/* === QScrollBar:horizontal === */
QScrollBar:horizontal {
    height: 16px;
    margin: 0 16px 0 16px;
    background: #333;
}

QScrollBar::handle:horizontal {
    background: #888;
    min-width: 16px;
    border-left: 1px solid #666;
    border-right: 1px solid #666;
}

QScrollBar::add-line:horizontal {
    background: #888;
    width: 16px;
    subcontrol-position: right;
    subcontrol-origin: margin;
}

QScrollBar::sub-line:horizontal {
    background: #888;
    width: 16px;
    subcontrol-position: left;
    subcontrol-origin: margin;
}

QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal {
    background: none;
}

/* === QMenu === */
QMenu {
    background-color: black;
    border: 1px solid gray;
    color: white;
    padding: 1px;
}

QMenu::item {
    padding: 2px 25px 2px 20px;
    border: 1px solid transparent;
}

QMenu::item:disabled {
    color: #666666;
}

QMenu::item:selected {
    border-color: gray;
    background: #222222;
}

QMenu::icon:checked {

}

QMenu::separator {
    height: 1px;
    background: #222222;
    margin-left: 10px;
    margin-right: 10px;
    margin-top: 1px;
    margin-bottom: 1px;
}

QMenu::indicator {
    width: 13px;
    height: 13px;
}

/* === QMenuBar === */
QMenuBar {
    background-color: black;
    color: white;
}

QMenuBar::item {
    background: transparent;
}

QMenuBar::item:disabled {
    color: gray;
}

QMenuBar::item:selected {
    background: #222222;
}

QMenuBar::item:pressed {
    background: #444444;
}
/* =================== */
QLineEdit, QListView, QTreeView, QTableView, QAbstractSpinBox {
    background-color: #AAA;
    color: #000;
    border: 1px solid #555;
}

QAbstractScrollArea, QLineEdit, QTextEdit, QAbstractSpinBox, QComboBox {
    border: 1px solid #555;
}

/* === QHeaderView === */
QHeaderView::section {
    height: 20px;
}

QHeaderView::section {
    background: #666;
    border: 0;
    color: #000;
    padding-left: 4px;
}

/* === QListView === */
QListView::item:hover {
    background: #AAA;
}

QListView::item:selected {
    background: #333;
    color: #AAA;
}

/* === QTableView === */
QTableView::item:hover {
    background: #333333;
}

QTableView::item:hover {
    background: #111111;
    color: antiquewhite;
}

/* === QTreeView === */
QTreeView::item {
    background: #AAA;
}

QTreeView::item:hover {
    background: #CCC;
}

QTreeView::item:selected {
    background: #333;
    color: #AAA;
}

QTreeView::branch {

}

QTreeView::branch:has-siblings:adjoins-item {

}

QTreeView::branch:has-siblings:!adjoins-item {

}

QTreeView::branch:closed:has-children:has-siblings {

}

QTreeView::branch:has-children:!has-siblings:closed {

}

QTreeView::branch:!has-children:!has-siblings:adjoins-item {

}

QTreeView::branch:open:has-children:has-siblings {

}

QTreeView::branch:open:has-children:!has-siblings {

}
"""


import logging, httpx, os, json, string, random
from .reader import readox
from .engine import process
from .writer import writedox
from .download import download

class WordCore(object):
    """[summary]
    Args:
        basepath : 工作区目录
        log_level : logging.DEBUG/INFO/WARNING
    """
    def __init__(self, basepath : str, log_level=logging.INFO):
        logging.basicConfig(level=log_level,
                    format='%(levelname)s: %(message)s')  
                    # format='%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s: %(message)s')  
        logging.info("""
##############################################
##                                          ##
##           Welcome to FastDocx!!          ##
##                                          ##
##        Author      sovlookup             ##
##        Mail     805408477@qq.com         ##
##        Github  sovlookup/FastDocx        ##
##                                          ##
##############################################
        """)
        if basepath.endswith("/") == False:
            basepath = basepath+"/"

        self.basepath = basepath
    
    def verify(self, config : dict or str, thread_num:int):
        logging.info("校验中...")
        if type(config) == str and config.startswith("http"):
            # 下载config.json
            config = json.loads(httpx.get(config).content)

        id = config.get("id")
        if id == None:
            logging.error("资源地址不正确或已失效！")
            return False
        
        word = config.get("word")
        if word == None:
            logging.error("资源无效！")
            return False
        
        self.id = id 

        self.readpath = self.basepath+self.id+"/tmp/"
        self.outpath = self.basepath+self.id+"/out/"

        # 检查项目资源;项目目录，没有就创建
        if os.path.exists(self.readpath) == False:
            os.makedirs(self.readpath)
        if os.path.exists(self.outpath) == False:
            os.makedirs(self.outpath)

        if os.path.exists(self.readpath+"template.docx") == False:
            tmplate_url = config.get("template",False)
            if tmplate_url == False:
                logging.error("没有找到工作资源！")
                return False
            if tmplate_url.strip().startswith("http") == False:
                logging.error("资源地址错误！")
                return False
            
            # 下载到 self.readpath+"template.docx"
            with httpx.stream("GET", tmplate_url) as response:
                with open(self.readpath+"template.docx","wb+") as f:
                    for chunk in response.iter_bytes():
                        f.write(chunk)

        if os.path.exists(self.readpath + "img/") == False:
            os.makedirs(self.readpath + "img/")

        # 下载img
        msg = [
        # {
        #     "content":["GET","https://baidu.com"],
        #     "todo": print("sssssssssssssssssssssssssss")
        # },
        ]
        for part in word:
            for section in part.get("content"):
                value = section.get("value")
                if type(value) == list:
                    if (type(value[0]) == str) and (value[0].startswith("http")):
                        # 下载并保存到readpath/img/ 返回名字
                        name = ''.join(random.sample(string.ascii_letters + string.digits, 12)) + ".png"
                        msg.append({
                            "content":["GET",value[0]],
                            "path": self.readpath + "img/" + name
                        })

                        value[0] = self.readpath + "img/"+name
        
        logging.info(f"同步资源...共{len(msg)}个项目")
        download(msg, thread_num)
        
        # # 保存config.json
        # with open(self.readpath+"config.json","w+") as f:
        #     f.write(json.dumps(config))

        self.word = word
        self.config = config

    def load(self, config : dict, thread_num:int = 1):
        """[summary]

        Args:
            config : config
            thread_num (int, optional): [下载线程数]. Defaults to 1.
        """

        if self.verify(config=config,thread_num=thread_num):
            logging.warning("加载失败！！！")
            return

        self.taskname = self.config.get("taskname","")
        self.author = self.config.get("author","")
        self.version = self.config.get("version","")
        self.description = self.config.get("description","")
        
        # 输出自述信息
        logging.info(f"""
        任务ID\t\t|\t{self.id}
        任务名称\t|\t{self.taskname}
        作者\t\t|\t{self.author}
        版本\t\t|\t{self.version}
        任务描述\t|\t{self.description}
        """)

        # 加载模板
        self.template = readox(self.readpath+"template.docx")

        return self

    def process(self) -> bool:
        logging.info("#"*16+f"{self.id}任务开始"+"#"*16)

        for word_content in self.word:
            name = word_content["name"]
            content = word_content["content"]

            logging.info("gen "+name)
            # 使用模板和内容组成新文档
            newdoc = process(self.template,content,self.readpath+"img/")

            # 保存新文档
            writedox(newdoc,self.outpath + name)


        logging.info("#"*16+f"{self.id}任务结束"+"#"*16)
        return True


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'form.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1280, 720)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setStyleSheet("")
        MainWindow.setAnimated(False)
        MainWindow.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setEnabled(True)
        self.centralwidget.setStyleSheet("")
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setContentsMargins(5, 5, 5, 5)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.listWidget = QtWidgets.QListWidget(self.tab)
        self.listWidget.setObjectName("listWidget")
        self.verticalLayout_3.addWidget(self.listWidget)
        self.groupBox = QtWidgets.QGroupBox(self.tab)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_7.addWidget(self.label_2)
        self.name = QtWidgets.QLabel(self.groupBox)
        self.name.setText("")
        self.name.setObjectName("name")
        self.horizontalLayout_7.addWidget(self.name)
        self.horizontalLayout_7.setStretch(0, 1)
        self.horizontalLayout_7.setStretch(1, 9)
        self.verticalLayout_5.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_9.addWidget(self.label_4)
        self.author = QtWidgets.QLabel(self.groupBox)
        self.author.setText("")
        self.author.setObjectName("author")
        self.horizontalLayout_9.addWidget(self.author)
        self.horizontalLayout_9.setStretch(0, 2)
        self.horizontalLayout_9.setStretch(1, 8)
        self.horizontalLayout_6.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_6 = QtWidgets.QLabel(self.groupBox)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_8.addWidget(self.label_6)
        self.version = QtWidgets.QLabel(self.groupBox)
        self.version.setText("")
        self.version.setObjectName("version")
        self.horizontalLayout_8.addWidget(self.version)
        self.horizontalLayout_8.setStretch(0, 2)
        self.horizontalLayout_8.setStretch(1, 8)
        self.horizontalLayout_6.addLayout(self.horizontalLayout_8)
        self.verticalLayout_5.addLayout(self.horizontalLayout_6)
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.verticalLayout_6.addWidget(self.label)
        self.description = QtWidgets.QLabel(self.groupBox)
        self.description.setText("")
        self.description.setObjectName("description")
        self.verticalLayout_6.addWidget(self.description)
        self.verticalLayout_6.setStretch(0, 1)
        self.verticalLayout_6.setStretch(1, 9)
        self.verticalLayout_5.addLayout(self.verticalLayout_6)
        self.verticalLayout_5.setStretch(0, 1)
        self.verticalLayout_5.setStretch(1, 1)
        self.verticalLayout_5.setStretch(2, 5)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.verticalLayout_3.setStretch(0, 7)
        self.verticalLayout_3.setStretch(1, 4)
        self.verticalLayout_2.addLayout(self.verticalLayout_3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.workdir = QtWidgets.QLineEdit(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.workdir.sizePolicy().hasHeightForWidth())
        self.workdir.setSizePolicy(sizePolicy)
        self.workdir.setObjectName("workdir")
        self.horizontalLayout_4.addWidget(self.workdir)
        self.workdirButton = QtWidgets.QToolButton(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.workdirButton.sizePolicy().hasHeightForWidth())
        self.workdirButton.setSizePolicy(sizePolicy)
        self.workdirButton.setObjectName("workdirButton")
        self.horizontalLayout_4.addWidget(self.workdirButton)
        self.horizontalLayout_2.addLayout(self.horizontalLayout_4)
        self.process = QtWidgets.QPushButton(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.process.sizePolicy().hasHeightForWidth())
        self.process.setSizePolicy(sizePolicy)
        self.process.setObjectName("process")
        self.horizontalLayout_2.addWidget(self.process)
        self.horizontalLayout_2.setStretch(0, 1)
        self.horizontalLayout_2.setStretch(1, 1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.verticalLayout_2.setStretch(0, 9)
        self.verticalLayout_2.setStretch(1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.tab_2)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.label_9 = QtWidgets.QLabel(self.tab_2)
        self.label_9.setObjectName("label_9")
        self.verticalLayout_7.addWidget(self.label_9)
        self.tabWidget.addTab(self.tab_2, "")
        self.horizontalLayout.addWidget(self.tabWidget)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menuBar = QtWidgets.QMenuBar(MainWindow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 1280, 18))
        self.menuBar.setDefaultUp(True)
        self.menuBar.setObjectName("menuBar")
        self.settings = QtWidgets.QMenu(self.menuBar)
        self.settings.setObjectName("settings")
        self.about = QtWidgets.QMenu(self.menuBar)
        self.about.setObjectName("about")
        MainWindow.setMenuBar(self.menuBar)
        self.source = QtWidgets.QAction(MainWindow)
        self.source.setObjectName("source")
        self.settings.addAction(self.source)
        self.menuBar.addAction(self.settings.menuAction())
        self.menuBar.addAction(self.about.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.groupBox.setTitle(_translate("MainWindow", "任务信息"))
        self.label_2.setText(_translate("MainWindow", "任务名称:"))
        self.label_4.setText(_translate("MainWindow", "作者:"))
        self.label_6.setText(_translate("MainWindow", "版本:"))
        self.label.setText(_translate("MainWindow", "详情:"))
        self.workdirButton.setText(_translate("MainWindow", "输出到"))
        self.process.setText(_translate("MainWindow", "开始任务"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("MainWindow", "部署"))
        self.label_9.setText(_translate("MainWindow", "功能开发中..."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("MainWindow", "提交"))
        self.settings.setTitle(_translate("MainWindow", "设置"))
        self.about.setTitle(_translate("MainWindow", "关于"))
        self.source.setText(_translate("MainWindow", "源地址"))



import httpx, trio
from tqdm import tqdm

async def run(reqs ,thread_num :int):
    """[summary]

    Args:
        thread_num (int, optional): [下载线程数]. Defaults to 1.
    """
    async with trio.open_nursery() as nursery:
        # Open a channel:
        # request_chan
        send_channel, receive_channel_ = trio.open_memory_channel(0)
        # response_chan
        send_channel_, receive_channel = trio.open_memory_channel(0)

        async with receive_channel_, send_channel_, send_channel, send_channel_:
            nursery.start_soon(req_chan, send_channel.clone(), reqs)
            # download
            for _ in range(thread_num):
                nursery.start_soon(downloader, receive_channel_.clone(), send_channel_.clone())
            nursery.start_soon(rep_chan, receive_channel.clone(),len(reqs))

async def downloader(receive_channel_,send_channel_):
    async with receive_channel_, send_channel_, httpx.AsyncClient() as client:
        async for msg in receive_channel_:
            msg["content"] = await client.send(msg["content"])
            await send_channel_.send(msg)

async def req_chan(send_channel, reqs):
    # Producer sends 3 messages
    async with send_channel:
        for msg in reqs:
            msg["content"] = httpx.Request(*msg["content"])
            
            await send_channel.send(msg)

# todo 将图片等写入本地
async def rep_chan(receive_channel, pbarnum):
    with tqdm(pbarnum) as pbar:
        async with receive_channel:
            # The consumer uses an 'async for' loop to receive the msgs:
            async for msg in receive_channel:
                # 保存
                with open(msg["path"],"wb+") as f:
                    f.write(msg["content"].content)

                pbar.update(1)

def download(reqs :list, thread_num :int = 1):
    """[summary]

    Args:
        reqs (msg): [description]
        {
        "content":["GET","https://v.gonorth.top:444/file/111111111111/img/2.png"],
        "path": "d:/1.png"
        },
        thread_num (int, optional): [description]. Defaults to 1.
    """
    trio.run(run,reqs,thread_num)

if __name__ == "__main__":
    mymsg = [
        {
            "content":["GET","https://v.gonorth.top:444/file/111111111111/img/2.png"],
            "path": "d:/test/1.png"
        },{
            "content":["GET","https://v.gonorth.top:444/file/111111111111/img/2.png"],
            "path": "d:/test/2.png"
        },{
            "content":["GET","https://v.gonorth.top:444/file/111111111111/img/2.png"],
            "path": "d:/test/3.png"
        },{
            "content":["GET","https://v.gonorth.top:444/file/111111111111/img/2.png"],
            "path": "d:/test/4.png"
        },
        ]

    download(mymsg,4)


import setuptools

with open("README.md", "r+", encoding="utf8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="fastdocx",
    version="0.1.6",
    author="sovlookup",
    author_email="805408477@qq.com",
    description="use template to gen word everywhere",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SOVLOOKUP/FastDocx",
    install_requires=[
        "python-docx",
        "trio",
        "httpx"
    ],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)